@@ -5,10 +5,11 @@
 
     Blueprints (and currently modules)
 
-    :copyright: © 2010 by the Pallets team.
-    :license: BSD, see LICENSE for more details.
+    :copyright: 2010 Pallets
+    :license: BSD-3-Clause
 """
 
+import functools
 import pytest
 
 import flask
@@ -114,17 +115,28 @@ def test_blueprint_app_error_handling(app, client):
     assert client.get('/nope').data == b'you shall not pass'
 
 
-def test_blueprint_prefix_slash(app, client):
-    bp = flask.Blueprint('test', __name__, url_prefix='/bar/')
+@pytest.mark.parametrize(('prefix', 'rule', 'url'), (
+    ('', '/', '/'),
+    ('/', '', '/'),
+    ('/', '/', '/'),
+    ('/foo', '', '/foo'),
+    ('/foo/', '', '/foo/'),
+    ('', '/bar', '/bar'),
+    ('/foo/', '/bar', '/foo/bar'),
+    ('/foo/', 'bar', '/foo/bar'),
+    ('/foo', '/bar', '/foo/bar'),
+    ('/foo/', '//bar', '/foo/bar'),
+    ('/foo//', '/bar', '/foo/bar'),
+))
+def test_blueprint_prefix_slash(app, client, prefix, rule, url):
+    bp = flask.Blueprint('test', __name__, url_prefix=prefix)
 
-    @bp.route('/foo')
-    def foo():
+    @bp.route(rule)
+    def index():
         return '', 204
 
     app.register_blueprint(bp)
-    app.register_blueprint(bp, url_prefix='/spam/')
-    assert client.get('/bar/foo').status_code == 204
-    assert client.get('/spam/foo').status_code == 204
+    assert client.get(url).status_code == 204
 
 
 def test_blueprint_url_defaults(app, client):
@@ -382,6 +394,8 @@ def test_route_decorator_custom_endpoint_with_dots(app, client):
         )
     )
 
+    bp.add_url_rule('/bar/456', endpoint='foofoofoo', view_func=functools.partial(foo_foo_foo))
+
     app.register_blueprint(bp, url_prefix='/py')
 
     assert client.get('/py/foo').data == b'bp.foo'
